import os

from aicage.config.project_config import AgentConfig
from aicage.paths import CONTAINER_DOCKER_SOCKET_PATH, HOST_DOCKER_SOCKET_PATH
from aicage.runtime.env_vars import DOCKER_HOST, WINDOWS_DOCKER_HOST
from aicage.runtime.prompts.confirm import prompt_persist_docker_socket
from aicage.runtime.run_args import EnvVar, MountSpec


def resolve_docker_socket_mount(
    agent_cfg: AgentConfig,
    cli_docker_socket: bool,
) -> tuple[list[MountSpec], list[EnvVar]]:
    mounts_cfg = agent_cfg.mounts
    docker_socket_enabled = cli_docker_socket or bool(mounts_cfg.docker)
    if not docker_socket_enabled:
        return [], []

    if os.name == "nt":
        mounts: list[MountSpec] = []
        env = [EnvVar(name=DOCKER_HOST, value=WINDOWS_DOCKER_HOST)]
    else:
        mounts = [
            MountSpec(
                host_path=HOST_DOCKER_SOCKET_PATH,
                container_path=CONTAINER_DOCKER_SOCKET_PATH,
            )
        ]
        env = []

    if cli_docker_socket and mounts_cfg.docker is None:
        if prompt_persist_docker_socket():
            mounts_cfg.docker = True

    return mounts, env



import os
from dataclasses import dataclass
from pathlib import Path

from aicage.config.agent.models import AgentMetadata


@dataclass
class AgentConfig:
    agent_path: list[str]
    agent_config_host: list[Path]


def resolve_agent_config(agent_metadata: AgentMetadata) -> AgentConfig:
    agent_paths = agent_metadata.agent_path
    agent_config_hosts = [_ensure_agent_path(path) for path in agent_paths]
    return AgentConfig(agent_path=agent_paths, agent_config_host=agent_config_hosts)


def _ensure_agent_path(agent_path: str) -> Path:
    expanded = Path(os.path.expanduser(agent_path)).resolve()
    if expanded.exists():
        return expanded
    if _looks_like_file(agent_path, expanded):
        expanded.parent.mkdir(parents=True, exist_ok=True)
        expanded.touch()
    else:
        expanded.mkdir(parents=True, exist_ok=True)
    return expanded


def _looks_like_file(agent_path: str, expanded: Path) -> bool:
    return bool(Path(agent_path).suffix)



from pathlib import PurePosixPath

from aicage.cli_types import ParsedArgs
from aicage.config.runtime_config import RunConfig
from aicage.paths import CONTAINER_AGENT_CONFIG_DIR
from aicage.runtime._agent_config import AgentConfig, resolve_agent_config
from aicage.runtime.run_args import DockerRunArgs, MountSpec, merge_docker_args


def build_run_args(config: RunConfig, parsed: ParsedArgs) -> DockerRunArgs:
    agent_config: AgentConfig = resolve_agent_config(
        config.context.agents[config.agent],
    )

    merged_docker_args: str = merge_docker_args(
        config.project_docker_args,
        parsed.docker_args,
    )
    agent_config_mounts = _build_agent_config_mounts(agent_config)
    return DockerRunArgs(
        image_ref=config.selection.image_ref,
        project_path=config.project_path,
        agent_config_mounts=agent_config_mounts,
        merged_docker_args=merged_docker_args,
        agent_args=parsed.agent_args,
        env=config.env,
        mounts=config.mounts,
    )


def _build_agent_config_mounts(agent_config: AgentConfig) -> list[MountSpec]:
    mounts: list[MountSpec] = []
    for agent_path, host_path in zip(agent_config.agent_path, agent_config.agent_config_host, strict=True):
        relative_path = _relative_agent_path(agent_path)
        container_path = CONTAINER_AGENT_CONFIG_DIR / relative_path
        mounts.append(MountSpec(host_path=host_path, container_path=container_path))
    return mounts


def _relative_agent_path(agent_path: str) -> PurePosixPath:
    if agent_path.startswith("~/"):
        return PurePosixPath(agent_path[2:])
    if agent_path.startswith("~\\"):
        return PurePosixPath(agent_path[2:].replace("\\", "/"))
    if agent_path.startswith("/"):
        return PurePosixPath(agent_path[1:])
    return PurePosixPath(agent_path)
